#!/usr/bin/perl
# Create, update or delete a backup

require './server-manager-lib.pl';
&error_setup($text{'backup_err'});
&error_if_cloudmin_replica();
&can_action_error(undef, "backup");
&ReadParse();
if (!$in{'new'}) {
	($backup) = grep { $_->{'id'} eq $in{'id'} } &list_available_backups();
	$backup || &error($text{'backup_egone'});
	}
else {
	# Create new empty object
	$backup = { };
	if ($access{'owner'}) {
		$backup->{'owner'} = $base_remote_user;
		}
	}

if ($in{'delete'}) {
	# Just remove this backup
	&delete_backup($backup);
	&webmin_log("delete", "backup", $backup->{'id'},
		    { 'dest' => $backup->{'dest'} });
	}
elsif ($in{'restore'}) {
	# Redirect to restore form
	&redirect("restore_form.cgi?id=$backup->{'id'}");
	return;
	}
elsif ($in{'run'}) {
	# Redirect to run backup
	&redirect("backup.cgi?id=$backup->{'id'}");
	return;
	}
elsif ($in{'logs'}) {
	# Redirect to logs for this backup
	&redirect("list_backuplogs.cgi?log=$backup->{'id'}");
	return;
	}
else {
	# Validate and store inputs
	@allservers = grep { &supports_backup($_) }
                &list_available_managed_servers_sorted();
	if ($in{'servers_def'}) {
		$backup->{'servers'} = '*';
		}
	else {
		%sids = map { $_, 1 } split(/\0/, $in{'servers'});
		@servers = grep { $sids{$_->{'id'}} } @allservers;
		@servers || &error($text{'backup_eservers'});
		$backup->{'servers'} = join(' ', map { $_->{'id'} } @servers);
		}
	
	$backup->{'shutdown'} = $in{'shutdown'};
	$backup->{'parallel'} = $in{'parallel'};

	# Parse destination
	if ($in{'dest_mode'} == -1) {
		# Use host system setting
		delete($backup->{'dest'});
		}
	elsif ($in{'dest_mode'} == 2) {
		# Some other system
		$dest = { 'mode' => 2 };
		$dest->{'path'} = $in{'vm2dir'};
		$dest->{'id'} = $in{'vm2'};
		$dest->{'path'} =~ /^\/\S+/ || &error($text{'backups_emode2'});
		$backup->{'dest'} = &join_backup_destination($dest);
		}
	else {
		# SSH or FTP
		$dest = { 'mode' => $in{'dest_mode'} };
		$pfx = $in{'dest_mode'} == 3 ? 'ssh' : 'ftp';
		$err = &parse_backup_ssh_dest($dest, $pfx, \%in);
		&error($err) if ($err);
		$backup->{'dest'} = &join_backup_destination($dest);
		}

	# Parse schedule
	$backup->{'enabled'} = $in{'enabled'};
	&virtualmin_ui_parse_cron_time("when", $backup, \%in);

	# Parse email settings
	if ($in{'emailother'}) {
		$in{'email'} =~ /^\S+\@\S+$/ ||
			&error($text{'backup_eemail'});
		$backup->{'email'} = $in{'email'};
		}
	else {
		delete($backup->{'email'});
		}
	if (!$access{'owner'}) {
		$backup->{'emailmaster'} = $in{'emailmaster'};
		}
	$backup->{'always'} = $in{'always'};
	if ($in{'purge_def'}) {
		delete($backup->{'purge'});
		}
	else {
		$backup->{'dest'} || &error($text{'backup_epurgehost'});
		($base, $date) = &extract_purge_path($backup->{'dest'});
		$base || &error($text{'backup_epurgedate'});
		$in{'purge'} =~ /^[0-9\.]+$/ && $in{'purge'} > 0 ||
			&error($text{'backup_epurge'});
		$backup->{'purge'} = $in{'purge'};
		}

	# Save pre and post commands
	$master = &get_managed_server(0);
	if (&can_action($master, "run")) {
		$backup->{'pre'} = $in{'pre'};
		$backup->{'pre_host'} = $in{'pre_host'};
		$backup->{'post'} = $in{'post'};
		$backup->{'post_host'} = $in{'post_host'};
		}
	
	# Update the backup
	&save_backup($backup);
	&webmin_log($in{'new'} ? "create" : "modify", "backup", $backup->{'id'},
		    { 'dest' => $backup->{'dest'} });
	}

&redirect("list_backups.cgi");

